package de.jottyfan.timetrack.spring.contact;

import java.util.Arrays;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import de.jottyfan.timetrack.db.contact.enums.EnumContacttype;

/**
 * 
 * @author henkej
 *
 */
@Controller
public class ContactController {
	private static final Logger LOGGER = LogManager.getLogger(ContactController.class);

	private final HttpServletRequest request;

	@Autowired
	private IContactService contactService;

	@Autowired
	public ContactController(HttpServletRequest request) {
		this.request = request;
	}

	@GetMapping("/logout")
	public String getLogout(HttpServletRequest request) throws ServletException {
		request.logout();
		return "redirect:/";
	}

	@GetMapping("/")
	public String getIndex() {
		return "public/index";
	}
	
	@ModelAttribute("currentUser")
	@ResponseBody
	public String getCurrentUser() {
		return contactService.getCurrentUser(request);
	}

	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/contact/list")
	public String getList(Model model) {
		List<ContactBean> list = contactService.getList();
		model.addAttribute("contactList", list);
		return "contact/list";
	}

	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/contact/add", method = RequestMethod.GET)
	public String toAdd(Model model) {
		return toItem(null, model);
	}

	@RolesAllowed("timetrack_user")
  @GetMapping("/contact/edit/{id}")
	public String toItem(@PathVariable Integer id, Model model) {
		ContactBean bean = contactService.getBean(id);
		if (bean == null) {
			bean = new ContactBean(); // the add case
		}
		model.addAttribute("contactBean", bean);
		model.addAttribute("types", Arrays.asList(EnumContacttype.values()));
		return "contact/item";
	}

	@RolesAllowed("timetrack_user")
	@RequestMapping(value = "/contact/upsert", method = RequestMethod.POST)
	public String doUpsert(Model model, @ModelAttribute ContactBean bean) {
		Integer amount = contactService.doUpsert(bean);
		return amount.equals(1) ? getList(model) : toItem(bean.getPk(), model);
	}

	@RolesAllowed("timetrack_user")
	@GetMapping(value = "/contact/delete/{id}")
	public String doDelete(@PathVariable Integer id, Model model) {
		Integer amount = contactService.doDelete(id);
		return amount.equals(1) ? getList(model) : toItem(id, model);
	}
}
